# Copyright (C) 2007 - 2010 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
# Bjoern Ricks <bjoern.ricks@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""PackageTrack for kde entperprise tag only builds"""

import logging

from treepkg.readconfig import convert_bool
import recipes.kde.enterprise.generic as enterprise

class PackageTrack(enterprise.PackageTrack):

    """In contrast to recipes.kde.enterprise.generic.PackageTrack
       this PackageTrack either builds trunk or newest tag"""

    revision_packager_cls = enterprise.RevisionPackager

    extra_config_desc = [("tags_url", str, ""),
                         ("tags_pattern", str, ""),
                         ("tags_subdir", str, ""),
                         ("tags_scm_type", str, None),
                         ("tags_rules_url", str, None),
                         ("tags_rules_pattern", str, ""),
                         ("tags_rules_subdir", str, ""),
                         ("tags_rules_scm_type", str, None),
                         ("fail_if_tag_not_found", convert_bool, "true")]

    def __init__(self, *args, **kw):
        self.fail_if_tag_not_found = kw.pop("fail_if_tag_not_found")
        super(PackageTrack, self).__init__(*args, **kw)

    def packager_for_new_revision(self):
        logging.info("Checking tags")
        self.tag_url = None
        self.use_tag_rules = self.found_tag_rules
        tag_url, tag_revision = self.tag_detector.newest_tag_revision()
        tag_rules_url, tag_rules_revision = self.tag_rules_detector.newest_tag_revision()
        logging.info("Found: %s: %s", tag_url, tag_revision)
        logging.info("Found rules: %s: %s", tag_rules_url, tag_rules_revision)
        if self.use_tag_rules and tag_rules_url is None:
            self.use_tag_rules = False
            logging.error("No rules found in tag but tag should be used to" \
                          " build package.")
            return
        if tag_url is not None:
            if not self.use_tag_rules:
                tag_rules_revision = self.rules_working_copy.last_changed_revision()
            revision = (tag_revision, tag_rules_revision)
            logging.info("New revision is %s", revision)
            if revision not in self.get_revision_numbers():
                logging.info("Revision %s has not been packaged yet",
                             revision)
                self.tag_url = tag_url
                self.tag_revision = tag_revision
                self.tag_rules_url = tag_rules_url
                self.tag_rules_revision = tag_rules_revision
                tag_date, tag_change_count = self.tag_detector.tag_pkg_parameters(tag_url)
                logging.info("Tag date was %s", tag_date)
                pkg_revision = (self.pkg_revision_template
                                % dict(pkg_revision=tag_change_count,
                                       rules_revision=revision[1]))
                return self.revision_packager_cls(self, tag=tag_url,
                                                  pkg_revision=pkg_revision,
                                                  pkg_date=tag_date,
                                                  *revision)
            else:
                logging.info("Revision %s has already been packaged.",
                             revision)
        
        else:
            if self.fail_if_tag_not_found:
                logging.error("Could not find tag for track %s", self.name)
                return
            # tag_url is not set -> build trunk
            # don't use rules from tags for trunk builds
            self.use_tag_rules = False
            return super(PackageTrack, self).new_revsision_packager()
